@@ -18,6 +18,11 @@
 #include "qgssinglebandgrayrenderer.h"
 #include "qgscontrastenhancement.h"
 #include "qgsrastertransparency.h"
+#include "qgscolorramplegendnode.h"
+#include "qgscolorramplegendnodesettings.h"
+#include "qgsreadwritecontext.h"
+#include "qgscolorrampimpl.h"
+
 #include <QDomDocument>
 #include <QDomElement>
 #include <QImage>
@@ -29,6 +34,7 @@ QgsSingleBandGrayRenderer::QgsSingleBandGrayRenderer( QgsRasterInterface *input,
   , mGrayBand( grayBand )
   , mGradient( BlackToWhite )
   , mContrastEnhancement( nullptr )
+  , mLegendSettings( std::make_unique< QgsColorRampLegendNodeSettings >() )
 {
 }
 
@@ -42,9 +48,15 @@ QgsSingleBandGrayRenderer *QgsSingleBandGrayRenderer::clone() const
   {
     renderer->setContrastEnhancement( new QgsContrastEnhancement( *mContrastEnhancement ) );
   }
+  renderer->setLegendSettings( mLegendSettings ? new QgsColorRampLegendNodeSettings( *mLegendSettings.get() ) : new QgsColorRampLegendNodeSettings() );
   return renderer;
 }
 
+Qgis::RasterRendererFlags QgsSingleBandGrayRenderer::flags() const
+{
+  return Qgis::RasterRendererFlag::InternalLayerOpacityHandling;
+}
+
 QgsRasterRenderer *QgsSingleBandGrayRenderer::create( const QDomElement &elem, QgsRasterInterface *input )
 {
   if ( elem.isNull() )
@@ -52,7 +64,7 @@ QgsRasterRenderer *QgsSingleBandGrayRenderer::create( const QDomElement &elem, Q
     return nullptr;
   }
 
-  int grayBand = elem.attribute( QStringLiteral( "grayBand" ), QStringLiteral( "-1" ) ).toInt();
+  const int grayBand = elem.attribute( QStringLiteral( "grayBand" ), QStringLiteral( "-1" ) ).toInt();
   QgsSingleBandGrayRenderer *r = new QgsSingleBandGrayRenderer( input, grayBand );
   r->readXml( elem );
 
@@ -61,7 +73,7 @@ QgsRasterRenderer *QgsSingleBandGrayRenderer::create( const QDomElement &elem, Q
     r->setGradient( WhiteToBlack );  // BlackToWhite is default
   }
 
-  QDomElement contrastEnhancementElem = elem.firstChildElement( QStringLiteral( "contrastEnhancement" ) );
+  const QDomElement contrastEnhancementElem = elem.firstChildElement( QStringLiteral( "contrastEnhancement" ) );
   if ( !contrastEnhancementElem.isNull() )
   {
     QgsContrastEnhancement *ce = new QgsContrastEnhancement( ( Qgis::DataType )(
@@ -69,6 +81,11 @@ QgsRasterRenderer *QgsSingleBandGrayRenderer::create( const QDomElement &elem, Q
     ce->readXml( contrastEnhancementElem );
     r->setContrastEnhancement( ce );
   }
+
+  auto legendSettings = std::make_unique< QgsColorRampLegendNodeSettings >();
+  legendSettings->readXml( elem, QgsReadWriteContext() );
+  r->setLegendSettings( legendSettings.release() );
+
   return r;
 }
 
@@ -79,19 +96,19 @@ void QgsSingleBandGrayRenderer::setContrastEnhancement( QgsContrastEnhancement *
 
 QgsRasterBlock *QgsSingleBandGrayRenderer::block( int bandNo, const QgsRectangle &extent, int width, int height, QgsRasterBlockFeedback *feedback )
 {
-  Q_UNUSED( bandNo );
+  Q_UNUSED( bandNo )
   QgsDebugMsgLevel( QStringLiteral( "width = %1 height = %2" ).arg( width ).arg( height ), 4 );
 
-  std::unique_ptr< QgsRasterBlock > outputBlock( new QgsRasterBlock() );
+  auto outputBlock = std::make_unique<QgsRasterBlock>();
   if ( !mInput )
   {
     return outputBlock.release();
   }
 
-  std::shared_ptr< QgsRasterBlock > inputBlock( mInput->block( mGrayBand, extent, width, height, feedback ) );
+  const std::shared_ptr< QgsRasterBlock > inputBlock( mInput->block( mGrayBand, extent, width, height, feedback ) );
   if ( !inputBlock || inputBlock->isEmpty() )
   {
-    QgsDebugMsg( QStringLiteral( "No raster data!" ) );
+    QgsDebugError( QStringLiteral( "No raster data!" ) );
     return outputBlock.release();
   }
 
@@ -111,29 +128,40 @@ QgsRasterBlock *QgsSingleBandGrayRenderer::block( int bandNo, const QgsRectangle
     alphaBlock = inputBlock;
   }
 
-  if ( !outputBlock->reset( Qgis::ARGB32_Premultiplied, width, height ) )
+  if ( !outputBlock->reset( Qgis::DataType::ARGB32_Premultiplied, width, height ) )
   {
     return outputBlock.release();
   }
 
-  QRgb myDefaultColor = NODATA_COLOR;
+  const QRgb myDefaultColor = renderColorForNodataPixel();
+  bool isNoData = false;
   for ( qgssize i = 0; i < ( qgssize )width * height; i++ )
   {
-    if ( inputBlock->isNoData( i ) )
+    double grayVal = inputBlock->valueAndNoData( i, isNoData );
+
+    if ( isNoData )
     {
       outputBlock->setColor( i, myDefaultColor );
       continue;
     }
-    double grayVal = inputBlock->value( i );
 
     double currentAlpha = mOpacity;
     if ( mRasterTransparency )
     {
-      currentAlpha = mRasterTransparency->alphaValue( grayVal, mOpacity * 255 ) / 255.0;
+      currentAlpha *= mRasterTransparency->opacityForValue( grayVal );
     }
     if ( mAlphaBand > 0 )
     {
-      currentAlpha *= alphaBlock->value( i ) / 255.0;
+      const double alpha = alphaBlock->value( i );
+      if ( alpha == 0 )
+      {
+        outputBlock->setColor( i, myDefaultColor );
+        continue;
+      }
+      else
+      {
+        currentAlpha *= alpha / 255.0;
+      }
     }
 
     if ( mContrastEnhancement )
@@ -164,6 +192,31 @@ QgsRasterBlock *QgsSingleBandGrayRenderer::block( int bandNo, const QgsRectangle
   return outputBlock.release();
 }
 
+void QgsSingleBandGrayRenderer::setGrayBand( int band )
+{
+  setInputBand( band );
+}
+
+int QgsSingleBandGrayRenderer::inputBand() const
+{
+  return mGrayBand;
+}
+
+bool QgsSingleBandGrayRenderer::setInputBand( int band )
+{
+  if ( !mInput )
+  {
+    mGrayBand = band;
+    return true;
+  }
+  else if ( band > 0 && band <= mInput->bandCount() )
+  {
+    mGrayBand = band;
+    return true;
+  }
+  return false;
+}
+
 void QgsSingleBandGrayRenderer::writeXml( QDomDocument &doc, QDomElement &parentElem ) const
 {
   if ( parentElem.isNull() )
@@ -193,18 +246,45 @@ void QgsSingleBandGrayRenderer::writeXml( QDomDocument &doc, QDomElement &parent
     mContrastEnhancement->writeXml( doc, contrastElem );
     rasterRendererElem.appendChild( contrastElem );
   }
+
+  if ( mLegendSettings )
+    mLegendSettings->writeXml( doc, rasterRendererElem, QgsReadWriteContext() );
+
   parentElem.appendChild( rasterRendererElem );
 }
 
-void QgsSingleBandGrayRenderer::legendSymbologyItems( QList< QPair< QString, QColor > > &symbolItems ) const
+QList<QPair<QString, QColor> > QgsSingleBandGrayRenderer::legendSymbologyItems() const
 {
+  QList<QPair<QString, QColor> >  symbolItems;
   if ( mContrastEnhancement && mContrastEnhancement->contrastEnhancementAlgorithm() != QgsContrastEnhancement::NoEnhancement )
   {
-    QColor minColor = ( mGradient == BlackToWhite ) ? Qt::black : Qt::white;
-    QColor maxColor = ( mGradient == BlackToWhite ) ? Qt::white : Qt::black;
+    const QColor minColor = ( mGradient == BlackToWhite ) ? Qt::black : Qt::white;
+    const QColor maxColor = ( mGradient == BlackToWhite ) ? Qt::white : Qt::black;
     symbolItems.push_back( qMakePair( QString::number( mContrastEnhancement->minimumValue() ), minColor ) );
     symbolItems.push_back( qMakePair( QString::number( mContrastEnhancement->maximumValue() ), maxColor ) );
   }
+  return symbolItems;
+}
+
+QList<QgsLayerTreeModelLegendNode *> QgsSingleBandGrayRenderer::createLegendNodes( QgsLayerTreeLayer *nodeLayer )
+{
+  QList<QgsLayerTreeModelLegendNode *> res;
+  if ( mContrastEnhancement && mContrastEnhancement->contrastEnhancementAlgorithm() != QgsContrastEnhancement::NoEnhancement )
+  {
+    const QString name = displayBandName( mGrayBand );
+    if ( !name.isEmpty() )
+    {
+      res << new QgsSimpleLegendNode( nodeLayer, name );
+    }
+
+    const QColor minColor = ( mGradient == BlackToWhite ) ? Qt::black : Qt::white;
+    const QColor maxColor = ( mGradient == BlackToWhite ) ? Qt::white : Qt::black;
+    res << new QgsColorRampLegendNode( nodeLayer, new QgsGradientColorRamp( minColor, maxColor ),
+                                       mLegendSettings ? *mLegendSettings : QgsColorRampLegendNodeSettings(),
+                                       mContrastEnhancement->minimumValue(),
+                                       mContrastEnhancement->maximumValue() );
+  }
+  return res;
 }
 
 QList<int> QgsSingleBandGrayRenderer::usesBands() const
@@ -217,10 +297,8 @@ QList<int> QgsSingleBandGrayRenderer::usesBands() const
   return bandList;
 }
 
-void QgsSingleBandGrayRenderer::toSld( QDomDocument &doc, QDomElement &element, const QgsStringMap &props ) const
+void QgsSingleBandGrayRenderer::toSld( QDomDocument &doc, QDomElement &element, const QVariantMap &props ) const
 {
-  QgsStringMap newProps = props;
-
   // create base structure
   QgsRasterRenderer::toSld( doc, element, props );
 
@@ -260,35 +338,36 @@ void QgsSingleBandGrayRenderer::toSld( QDomDocument &doc, QDomElement &element,
 
   // set band
   QDomElement sourceChannelNameElem = doc.createElement( QStringLiteral( "sld:SourceChannelName" ) );
-  sourceChannelNameElem.appendChild( doc.createTextNode( QString::number( grayBand() ) ) );
+  sourceChannelNameElem.appendChild( doc.createTextNode( QString::number( mGrayBand ) ) );
   channelElem.appendChild( sourceChannelNameElem );
 
   // set ContrastEnhancement
-  if ( contrastEnhancement() )
+  if ( auto *lContrastEnhancement = contrastEnhancement() )
   {
     QDomElement contrastEnhancementElem = doc.createElement( QStringLiteral( "sld:ContrastEnhancement" ) );
-    contrastEnhancement()->toSld( doc, contrastEnhancementElem );
+    lContrastEnhancement->toSld( doc, contrastEnhancementElem );
 
     // do changes to minValue/maxValues depending on stretching algorithm. This is necessary because
-    // geoserver do a first stretch on min/max, then apply colo map rules. In some combination is necessary
-    // to use real min/max values and in othere the actual edited min/max values
-    switch ( contrastEnhancement()->contrastEnhancementAlgorithm() )
+    // geoserver does a first stretch on min/max, then applies color map rules.
+    // In some combination it is necessary to use real min/max values and in
+    // others the actual edited min/max values
+    switch ( lContrastEnhancement->contrastEnhancementAlgorithm() )
     {
       case QgsContrastEnhancement::StretchAndClipToMinimumMaximum:
       case QgsContrastEnhancement::ClipToMinimumMaximum:
       {
         // with this renderer export have to be check against real min/max values of the raster
-        QgsRasterBandStats myRasterBandStats = mInput->bandStatistics( grayBand(), QgsRasterBandStats::Min | QgsRasterBandStats::Max );
+        const QgsRasterBandStats myRasterBandStats = mInput->bandStatistics( mGrayBand, Qgis::RasterBandStatistic::Min | Qgis::RasterBandStatistic::Max );
 
         // if minimum range differ from the real minimum => set is in exported SLD vendor option
-        if ( !qgsDoubleNear( contrastEnhancement()->minimumValue(), myRasterBandStats.minimumValue ) )
+        if ( !qgsDoubleNear( lContrastEnhancement->minimumValue(), myRasterBandStats.minimumValue ) )
         {
           // look for VendorOption tag to look for that with minValue attribute
-          QDomNodeList elements = contrastEnhancementElem.elementsByTagName( QStringLiteral( "sld:VendorOption" ) );
-          for ( int i = 0; i < elements.size(); ++i )
+          const QDomNodeList vendorOptions = contrastEnhancementElem.elementsByTagName( QStringLiteral( "sld:VendorOption" ) );
+          for ( int i = 0; i < vendorOptions.size(); ++i )
           {
-            QDomElement vendorOption = elements.at( i ).toElement();
-            if ( vendorOption.attribute( QStringLiteral( "name" ) ) != QStringLiteral( "minValue" ) )
+            QDomElement vendorOption = vendorOptions.at( i ).toElement();
+            if ( vendorOption.attribute( QStringLiteral( "name" ) ) != QLatin1String( "minValue" ) )
               continue;
 
             // remove old value and add the new one
@@ -311,8 +390,7 @@ void QgsSingleBandGrayRenderer::toSld( QDomDocument &doc, QDomElement &element,
 
   // for each color set a ColorMapEntry tag nested into "sld:ColorMap" tag
   // e.g. <ColorMapEntry color="#EEBE2F" quantity="-300" label="label" opacity="0"/>
-  QList< QPair< QString, QColor > > classes;
-  legendSymbologyItems( classes );
+  QList< QPair< QString, QColor > > classes = legendSymbologyItems();
 
   // add ColorMap tag
   QDomElement colorMapElem = doc.createElement( QStringLiteral( "sld:ColorMap" ) );
@@ -331,10 +409,10 @@ void QgsSingleBandGrayRenderer::toSld( QDomDocument &doc, QDomElement &element,
     case ( QgsContrastEnhancement::StretchAndClipToMinimumMaximum ):
     case ( QgsContrastEnhancement::ClipToMinimumMaximum ):
     {
-      QString lowValue = classes[0].first;
+      const QString lowValue = classes[0].first;
       QColor lowColor = classes[0].second;
       lowColor.setAlpha( 0 );
-      QString highValue = classes[1].first;
+      const QString highValue = classes[1].first;
       QColor highColor = classes[1].second;
       highColor.setAlpha( 0 );
 
@@ -355,8 +433,7 @@ void QgsSingleBandGrayRenderer::toSld( QDomDocument &doc, QDomElement &element,
   }
 
   // create tags
-  QList< QPair< QString, QColor > >::ConstIterator it;
-  for ( it = colorMapping.begin(); it != colorMapping.constEnd() ; ++it )
+  for ( auto it = colorMapping.constBegin(); it != colorMapping.constEnd() ; ++it )
   {
     // set low level color mapping
     QDomElement lowColorMapEntryElem = doc.createElement( QStringLiteral( "sld:ColorMapEntry" ) );
@@ -369,3 +446,35 @@ void QgsSingleBandGrayRenderer::toSld( QDomDocument &doc, QDomElement &element,
     }
   }
 }
+
+const QgsColorRampLegendNodeSettings *QgsSingleBandGrayRenderer::legendSettings() const
+{
+  return mLegendSettings.get();
+}
+
+void QgsSingleBandGrayRenderer::setLegendSettings( QgsColorRampLegendNodeSettings *settings )
+{
+  if ( settings == mLegendSettings.get() )
+    return;
+  mLegendSettings.reset( settings );
+}
+
+bool QgsSingleBandGrayRenderer::refresh( const QgsRectangle &extent, const QList<double> &min, const QList<double> &max, bool forceRefresh )
+{
+  if ( !needsRefresh( extent ) && !forceRefresh )
+  {
+    return false;
+  }
+
+  bool refreshed = false;
+  if ( mContrastEnhancement && mContrastEnhancement->contrastEnhancementAlgorithm() != QgsContrastEnhancement::NoEnhancement &&
+       min.size() >= 1 && max.size() >= 1 )
+  {
+    mLastRectangleUsedByRefreshContrastEnhancementIfNeeded = extent;
+    mContrastEnhancement->setMinimumValue( min[0] );
+    mContrastEnhancement->setMaximumValue( max[0] );
+    refreshed = true;
+  }
+
+  return refreshed;
+}
